﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

if not nUI_DefaultConfig then nUI_DefaultConfig = {}; end

local CreateFrame           = CreateFrame;
local GetNumShapeshiftForms = GetNumShapeshiftForms;

-------------------------------------------------------------------------------

nUI_DefaultConfig.SpecialBars =
{
	enabled = true;
	
	anchor =
	{
		anchor_pt   = "BOTTOM",
		relative_to = "nUI_Dashboard",
		relative_pt = "CENTER",
		xOfs        = 0,
		yOfs        = 187,
	},
	options =
	{
		enabled  = true,
		strata   = nil,
		level    = 2,
		btn_size = 36,
		btn_gap  = 1,
		inset    = 3,
		
		border =
		{
			backdrop =
			{
				bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
				edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
				tile     = true, 
				tileSize = 1, 
				edgeSize = 12, 
				insets   = {left = 0, right = 0, top = 0, bottom = 0},
			},
			border_color = { r = 1, g = 1, b = 1, a = 1 },
			backdrop_color = { r = 0, g = 0, b = 0, a = 0.35 },
		}
	},
};

-------------------------------------------------------------------------------

local anchor     = CreateFrame( "Frame", "nUI_SpecialBarEvents", WorldFrame );
local background = CreateFrame( "Frame", "nUI_SpecialBarsBackground", nUI_Dashboard.Anchor );
local frame      = CreateFrame( "Frame", "nUI_SpecialBars", nUI_Dashboard.Anchor, "SecureHandlerStateTemplate" );

background:SetAllPoints( frame );

-------------------------------------------------------------------------------

frame.applyScale = function( scale )
	
	local anchor  = scale and frame.anchor or nil;
	local scale   = scale or frame.scale or 1;
	local options = frame.options;
	
	if options then
		
		local btn_size = options.btn_size * scale * nUI.scale;
		local btn_gap  = (options.btn_gap or 0) * scale * nUI.scale;		
		local inset    = (options.inset or 0) * scale * nUI.scale;
		local bars     = 0;
	
		if frame.btn_size ~= btn_size 
		or frame.btn_gap  ~= btn_gap
		or frame.inset    ~= inset
		then
			
			frame.btn_size = btn_size;
			frame.btn_gap  = btn_gap;
			frame.inset    = inset;
				
			frame:SetAttribute( "nUI_inset", inset );
			frame:SetAttribute( "nUI_btnsize", btn_size );
			frame:SetAttribute( "nUI_btngap", btn_gap );
			
			nUI_PetBar.applyScale( scale );
			nUI_PossessBar.applyScale( scale );
			nUI_ShapeshiftBar.applyScale( scale );
			
		end

		frame.layoutBar();
		
	end
	
	if anchor then frame.applyAnchor(); end
	
end

-------------------------------------------------------------------------------

frame.applyAnchor = function( anchor )
	
	local anchor      = anchor or frame.anchor or {};
	local anchor_pt   = anchor.anchor_pt or "CENTER";
	local relative_to = anchor.relative_to or frame:GetParent():GetName();
	local relative_pt = anchor.relative_pt or anchor_pt;
	local xOfs        = (anchor.xOfs or 0) * nUI.scale;
	local yOfs        = (anchor.yOfs or 0) * nUI.scale;
	
	if frame.anchor_pt   ~= anchor_pt
	or frame.relative_to ~= relative_to
	or frame.relative_pt ~= relative_pt
	or frame.xOfs        ~= xOfs
	or frame.yOfs        ~= yOfs
	then
		
		frame.anchor_pt   = anchor_pt;
		frame.relative_to = relative_to;
		frame.relative_pt = relative_pt;
		frame.xOfs        = xOfs;
		frame.yOfs        = yOfs;
		
		frame:ClearAllPoints();
		frame:SetPoint( anchor_pt, relative_to, relative_pt, xOfs, yOfs );
		
	end
end

-------------------------------------------------------------------------------

frame.applyOptions = function( options )

	frame.options = options;
	
	if not options or not options.enabled then
		
		nUI_PetBar.applyOptions( nil );
		nUI_ShapeshiftBar.applyOptions( nil );
		nUI_PossessBar.applyOptions( nil );
		
		frame.enabled = false;
		UnregisterStateDriver( frame, "specialbars" );
		frame:Hide();
		
	else

		background:SetFrameStrata( options.strata or frame:GetParent():GetFrameStrata() );
		background:SetFrameLevel( options.level or frame:GetParent():GetFrameLevel()+1 );
		
		frame:SetFrameStrata( background:GetFrameStrata() );
		frame:SetFrameLevel( background:GetFrameLevel()+1 );

		nUI_PetBar.applyOptions( frame.options );
		nUI_ShapeshiftBar.applyOptions( frame.options );
		nUI_PossessBar.applyOptions( frame.options );
		
		frame.enabled = true;
	
		frame.applyScale( options.scale or frame.scale or 1 );
		
		if options.border then
			
			local border_color   = options.border.border_color;
			local backdrop_color = options.border.backdrop_color;
			
			frame:SetBackdrop( options.border.backdrop );
			frame:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
			frame:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			
		end
		
		if options.background then
			
			local border_color   = options.background.border_color;
			local backdrop_color = options.background.backdrop_color;
			
			background:SetBackdrop( options.background.backdrop );
			background:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
			background:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			
		end

		RegisterStateDriver( frame, "specialbars", "[bonusbar:5] 1; [pet] 2; 3" );
		
	end
end

-------------------------------------------------------------------------------

frame.applySkin = function( skin )
	
	frame.skin = skin.SpecialBars or nUI_DefaultConfig.SpecialBars;
	
	if not frame.skin or not frame.skin.enabled then
		
		frame.enabled = false;		
		UnregisterStateDriver( frame, "specialbars" );
		frame:Hide();
	
	else
		
		frame.enabled = true;
		
		frame.applyOptions( frame.skin.options );
		frame.applyAnchor( frame.skin.anchor );
		
	end
end

-------------------------------------------------------------------------------

frame.layoutBar = function()

	if frame.options and frame.options.enabled and not InCombatLockdown() then
		
		-- if there's a possession bar, then everything else is hidden
		
		local inset = frame.inset;
		local gap   = frame.btn_gap;
		local size  = frame.btn_size;
		local forms = GetNumShapeshiftForms();
		local width;
	
		frame:SetAttribute( "nUI_numforms", forms );
		
		if nUI_PossessBar:IsShown() then
			
			nUI_PetBar:Hide();
			nUI_ShapeshiftBar:Hide();
		
			nUI_PossessBar:Show();
			nUI_PossessBar:ClearAllPoints();
			nUI_PossessBar:SetPoint( "CENTER", frame, "CENTER", 0, 0 );

			width = nUI_PossessBar:GetWidth();
			
		-- otherwise, if there's a pet then we have to show at least the pet
		-- bar and possibly a shapeshift bar
		
		elseif nUI_PetBar:IsShown() then
								
			if forms > 0 then
				
				nUI_PossessBar:Hide();
				
				nUI_ShapeshiftBar:Show();
				nUI_ShapeshiftBar:ClearAllPoints();
				nUI_ShapeshiftBar:SetPoint( "TOPLEFT", frame, "TOPLEFT", inset, inset );
				
				nUI_PetBar:Show();
				nUI_PetBar:ClearAllPoints();
				nUI_PetBar:SetPoint( "LEFT", nUI_ShapeshiftBar, "RIGHT", gap, 0 );

				width = nUI_ShapeshiftBar:GetWidth() + nUI_PetBar:GetWidth();
				
			else
				
				nUI_PossessBar:Hide();
				nUI_ShapeshiftBar:Hide();
				
				nUI_PetBar:Show();
				nUI_PetBar:ClearAllPoints();
				nUI_PetBar:SetPoint( "TOPLEFT", frame, "TOPLEFT", inset, inset );
				
				width = nUI_PetBar:GetWidth();
				
			end
			
		-- otherwise, if the player has shapeshift forms, display that bar
		-- and hide the rest. If they have no forms, hide everything
		
		else

			nUI_PossessBar:Hide();
			nUI_PetBar:Hide();
			
			if forms > 0 then
				
				nUI_ShapeshiftBar:Show();
				nUI_ShapeshiftBar:ClearAllPoints();
				nUI_ShapeshiftBar:SetPoint( "TOPLEFT", frame, "TOPLEFT", inset, inset );
				
				width = nUI_ShapeshiftBar:GetWidth();
				
			else
				
				nUI_ShapeshiftBar:Hide();
				
				width = 0;
				
			end
			
		end
		
		-- set the size of the special bar frame
		
		if width == 0 then
			frame:Hide();
		else
			frame:SetWidth( width + inset * 2 );
			frame:SetHeight( size + inset * 2 );
			frame:Show();
		end
	end	
end

-------------------------------------------------------------------------------
-- manage the layout of the bar

anchor:SetScript( "OnEvent", 

	function() 

		-- initializs the skinner
		
		if event == "VARIABLES_LOADED" then
			
			nUI:registerScalableFrame( frame );
			nUI:registerSkinnedFrame( frame );

			frame:SetFrameRef( "nUI_PetBar", nUI_PetBar );
			frame:SetFrameRef( "nUI_ShapeshiftBar", nUI_ShapeshiftBar );
			frame:SetFrameRef( "nUI_PossessBar", nUI_PossessBar );
			
			frame:Execute(
				[[		
					nUI_PetBar         = self:GetFrameRef( "nUI_PetBar" );
					nUI_PossessBar     = self:GetFrameRef( "nUI_PossessBar" );
					nUI_ShapeshiftBar  = self:GetFrameRef( "nUI_ShapeshiftBar" );
				]]
			);
			
			frame:SetAttribute( 
				"_onstate-specialbars", 
				[[
					-- if there's a possession bar, then everything else is hidden
					
					frame = self;
					inset = frame:GetAttribute( "nUI_inset" ) or 0;
					gap   = frame:GetAttribute( "nUI_btngap" ) or 0;
					size  = frame:GetAttribute( "nUI_btnsize" ) or 39;
					forms = frame:GetAttribute( "nUI_numforms" ) or 0;
					
					if newstate == "1" then
						
						nUI_PetBar:Hide();
						nUI_ShapeshiftBar:Hide();
					
						nUI_PossessBar:Show();
						nUI_PossessBar:ClearAllPoints();
						nUI_PossessBar:SetPoint( "CENTER", frame, "CENTER", 0, 0 );

						width = nUI_PossessBar:GetWidth();
						
					-- otherwise, if there's a pet then we have to show at least the pet
					-- bar and possibly a shapeshift bar
					
					elseif newstate == "2" then
											
						if forms > 0 then
							
							nUI_PossessBar:Hide();
							
							nUI_ShapeshiftBar:Show();
							nUI_ShapeshiftBar:ClearAllPoints();
							nUI_ShapeshiftBar:SetPoint( "TOPLEFT", frame, "TOPLEFT", inset, inset );
							
							nUI_PetBar:Show();
							nUI_PetBar:ClearAllPoints();
							nUI_PetBar:SetPoint( "LEFT", nUI_ShapeshiftBar, "RIGHT", gap, 0 );

							width = nUI_ShapeshiftBar:GetWidth() + nUI_PetBar:GetWidth();
							
						else
							
							nUI_PossessBar:Hide();
							nUI_ShapeshiftBar:Hide();
							
							nUI_PetBar:Show();
							nUI_PetBar:ClearAllPoints();
							nUI_PetBar:SetPoint( "TOPLEFT", frame, "TOPLEFT", inset, inset );
							
							width = nUI_PetBar:GetWidth();
							
						end
						
					-- otherwise, if the player has shapeshift forms, display that bar
					-- and hide the rest. If they have no forms, hide everything
					
					else

						nUI_PossessBar:Hide();
						nUI_PetBar:Hide();
						
						if forms > 0 then
							
							nUI_ShapeshiftBar:Show();
							nUI_ShapeshiftBar:ClearAllPoints();
							nUI_ShapeshiftBar:SetPoint( "TOPLEFT", frame, "TOPLEFT", inset, inset );
							
							width = nUI_ShapeshiftBar:GetWidth();
							
						else
							
							nUI_ShapeshiftBar:Hide();
							
							width = 0;
							
						end
						
					end
					
					-- set the size of the special bar frame
					
					if width == 0 then
						frame:Hide();
					else
						frame:SetWidth( width + inset * 2 );
						frame:SetHeight( size + inset * 2 );
						frame:Show();
					end
				]]
			);        
					
		else

			frame.layoutBar();
			
		end
	end 
);

anchor:RegisterEvent( "VARIABLES_LOADED" );
anchor:RegisterEvent( "PET_BAR_UPDATE" );
anchor:RegisterEvent( "PLAYER_CONTROL_GAINED" );
anchor:RegisterEvent( "PLAYER_CONTROL_LOST" );
anchor:RegisterEvent( "PLAYER_ENTERING_WORLD" );
anchor:RegisterEvent( "PLAYER_FARSIGHT_FOCUS_CHANGED" );
anchor:RegisterEvent( "UPDATE_BONUS_ACTIONBAR" );
anchor:RegisterEvent( "UPDATE_SHAPESHIFT_FORMS" );
anchor:RegisterEvent( "UNIT_PET" );
anchor:RegisterEvent( "UNIT_FLAGS" );
